<?php

/**
 * @file
 * Administration form for Brandfolder.
 */

use Brandfolder\Brandfolder;

/**
 * Builds the Settings form.
 */
function brandfolder_settings() {
  $form = array();

  $default_brandfolder = variable_get('brandfolder_default_brandfolder');
  $default_collection = variable_get('brandfolder_default_collection');
  $brandfolders_list = $collections_list = [];
  $api_key = variable_get('brandfolder_guest_api_key', FALSE);
  if ($api_key) {
    $bf = new Brandfolder($api_key);
    try {
      $brandfolders_list = $bf->getBrandfolders();
    }
    catch (Exception $e) {
      drupal_set_message(t('After you enter an API key, you can select a default Brandfolder'));
    }
    try {
      $collections_list = $default_brandfolder ? $bf->getCollectionsInBrandfolder($default_brandfolder) : [];
    }
    catch (Exception $e) {
      drupal_set_message(t('After you choose a default Brandfolder, you can select a default collection if you wish'));
    }
  }

  /************************************
   * Credentials
   ************************************/
  $form['brandfolder_credentials'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Brandfolder Credentials'),
    '#collapsible' => TRUE,
    '#collapsed'   => variable_get('brandfolder_guest_api_key'),
  );

  // @todo: Use separate Admin and/or Collaborator account API key if/when we add support for writing files from Drupal to Brandfolder (or generic ability to map Drupal roles to Brandfolder roles to customize access, etc.).
  $form['brandfolder_credentials']['brandfolder_guest_api_key'] = [
    '#type'          => 'textfield',
    '#title'         => t('Brandfolder API Key'),
    '#description'   => t('The API key from a Brandfolder user account that has access to the Brandfolder/collection where your images will be stored. For security reasons, it is recommended to create an account with the "Guest" role in Brandfolder and use the API key from that account.'),
    '#maxlength'     => 256,
    '#default_value' => variable_get('brandfolder_guest_api_key'),
  ];

  /************************************
   * Basic Configuration
   ************************************/
  // @todo: which org/brandfolder/collection(s) to use, etc. Other media browser-related settings and Drupal role mapping?
  $form['basic'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Basic Configuration Options'),
    '#collapsible' => TRUE,
    '#collapsed'   => FALSE,
  );

  $form['basic']['brandfolder_default_brandfolder'] = [
    '#type'          => 'select',
    '#title'         => t('Default Brandfolder'),
    '#options'       => $brandfolders_list,
    '#default_value' => $default_brandfolder,
    '#description'   => t('The Brandfolder to use for all operations unless otherwise specified.'),
  ];

  $form['basic']['brandfolder_default_collection'] = [
    '#type'          => 'select',
    '#title'         => t('Default Collection'),
    '#options'       => $collections_list,
    '#default_value' => $default_collection,
    '#description'   => t('The collection to use for all operations unless otherwise specified.'),
  ];

  /************************************
   * CDN Configuration
   ************************************/
  $form['cdn'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('CDN Options'),
    '#collapsible' => TRUE,
    '#collapsed'   => FALSE,
  );

  $form['cdn']['brandfolder_default_cdn_file_format'] = [
    '#type'          => 'select',
    '#title'         => t('Default Image File Format'),
    // @codingStandardsIgnoreStart
    '#options'       => [
      'auto' => t('AUTO'),
      'jpg'  => 'JPG',
      'gif'  => 'GIF',
      'pdf'  => 'PDF',
      'png'  => 'PNG',
      'webp' => 'WEBP',
      'svg'  => 'SVG',
    ],
    // @codingStandardsIgnoreEnd
    '#default_value' => variable_get('brandfolder_default_cdn_file_format', 'jpg'),
    '#description'   => t('The format to use when generating CDN URLs for Brandfolder assets.'),
  ];

  /************************************
   * Webhooks
   ************************************/
  $form['webhooks'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Webhooks'),
    '#collapsible' => TRUE,
    '#collapsed'   => FALSE,
  );

  $form['webhooks']['brandfolder_webhooks_debug'] = [
    '#type'          => 'checkbox',
    '#title'         => t('Log all webhook events (e.g. for debugging purposes)'),
    '#default_value' => variable_get('brandfolder_webhooks_debug', FALSE),
  ];

  $form = system_settings_form($form);

  // Display some images from the selected Brandfolder/collection if applicable.
  if ($default_brandfolder) {
    $bf->default_brandfolder_id = $default_brandfolder;
    if ($default_collection) {
      $assets = $bf->listAssets([], $default_collection);
    }
    else {
      $assets = $bf->listAssets();
    }
    if ($assets) {
      $thumbnails = array_map(function ($asset) {
        return '<img src="' . $asset->attributes->thumbnail_url . '">';
      }, $assets->data);

      $form['sample_pics'] = [
        '#type' => 'markup',
        '#prefix' => '<h2>Sample Images</h2>',
        '#markup' => implode(' ', $thumbnails),
      ];
    }
  }

  return $form;
}
